//
// bidwatcher
// copyright (c) 1999, 2000, 2001, 2002
// Trent McNair (trent@rmci.net)
// Tom McNair  (tmcnair@cyberhighway.net)
// Wayne Schlitt (wayne@midwestcs.com)
// Ben Byer (bushing@users.sourceforge.net)
// Kevin Dwyer (kevin@pheared.net)
// 
// use of this code is restricted to the terms
// of the GNU GPL, which should have been included in this
// distribution. If not, see www.gnu.org/copyleft/gpl.html.
// Here is the short version:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//  helpers.cpp:
//
//  Helper functions for bidwatcher, no platform/gui specific
//  code.
// 

#include <stdio.h>
#include <stdlib.h>
#include <strstream.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <sys/time.h>
#include <gtk/gtk.h>
#include "bidwatcher.h"

char *CheckPrice="0123456789.$,";      // Characters allowed in a price   

char *SearchOps[2][15]={{ "Starts at",           // 1:  +1
			  "Currently",           // 2:  +1
			  "First bid",           // 3:  +1
			  "Quantity",            // 4:  +1
			  "# of bids",           // 5:  +1
			  "Time left",           // 6:  +1
			  "Location",            // 7:  +1
			  "Started",             // 8:  +1
			  "Ends",                // 9:  +1
			  "Seller (Rating)",     // 10: +1
			  "High bid",            // 11: +1
			  "Bid increment",       // 12: +1
			  "This is a ",          // 13: +1
			  "Lowest",
			  "*"},
			/* eBay Motors */
			{ "Currently",
			  "Time left ",
			  "Seller (rating) ",
			  "High bid ",
			  "# of bids ",
			  "Location ",
			  "Started ",
			  "ends ",
			  "Starts at",
			  "*"
			}
};

float calculateBidIncrement(float currentBid,  char *currency)
{
	// step 1 - parse currentBid into useable int's
  /*
  int err;
	char curBid[15];
 	if ( currentBid[0] == '$' )
	{
		int limit = strlen(currentBid);
		for ( int i=0; i < limit; i++ )
		{
			// strip the '$' off the currentBid;
			curBid[i] = currentBid[i + 1];
		}
		curBid[14] = '\0'; // because I'm insane
	}
	else
	{
		strcpy(newBid, "no bids");
		return;
		} */
	float fcb = currentBid;   // fcb = 'float current bid'
	float inc;
	switch(currency[0]) {
	case '$': // all three types of dollars have the same increments
	  // it's official! :)
	case 'U': // US dollar
	case 'A': // australian dollars
	case 'C': // canadian dollars
	  if ( fcb < 1.00 ) inc = 0.05;
	  else if ( fcb < 5.00 ) inc = 0.25;
	  else if ( fcb < 25.00 ) inc = 0.50;
	  else if ( fcb < 100.00 ) inc = 1.00;
	  else if ( fcb < 250.00 ) inc = 2.50;
	  else if ( fcb < 500.00 ) inc = 5.00;
	  else if ( fcb < 1000.00 ) inc = 10.00;
	  else if ( fcb < 2500.00 ) inc = 25.00;
	  else if ( fcb < 5000.00 ) inc = 50.00;
	  else inc = 100.00;
	  break;
	case 'G': // great british pounds
	  inc=100;   
	  if(fcb<3000) inc=50;   
	  if(fcb<1500) inc=20;   
	  if(fcb<600) inc=10;   
	  if(fcb<300)  inc=5;
	  if(fcb<150) inc=2;
	  if(fcb<60) inc=1;
	  if(fcb<15) inc=0.50;
	  if(fcb<5.00) inc=0.20;
	  if(fcb<1) inc=0.05;
	  break;
	case 'J': // japanese yen
	  inc=12000;   
	  if(fcb<=600000) inc=7500;   
	  if(fcb<=300000) inc=3000;   
	  if(fcb<=120000) inc=1200;   
	  if(fcb<=60000)  inc=600;
	  if(fcb<=30000) inc=240;
	  if(fcb<=12000) inc=120;
	  if(fcb<=3000) inc=60;
	  if(fcb<=600) inc=30;
	  if(fcb<=120) inc=5;
	  break;
	case 'D':  // deutsche marks (german)
	  inc=100;
	  if(fcb<10000) inc=10;
	  if(fcb<1000) inc=5;
	  if(fcb<100) inc=1;
	  break;
	default: 
	  inc=0;
	}
	fcb = fcb + inc;
	return fcb;
}

///////////////////////////////////////////////////////////////////////////////
// StripHtmlTags()
//    -strips the tags off an html string
// Thanks to Bob Beaty!
///////////////////////////////////////////////////////////////////////////////
char * StripHtmlTags(const char * stringToStrip)
{
  char *Buff=(char *)malloc(strlen(stringToStrip));
  Buff[0]='\0';
  int buffLength = strlen(stringToStrip);
  int BuffIdx = 0;
  int u,c;
  int IncludeFlag = 5; // strip up everything before the first tag, too
  // strip the html tags
  for (u = 0; u < buffLength; u++) {
    c = stringToStrip[u];
    if (c=='<') IncludeFlag=5;
    else if (c=='>') IncludeFlag=10;
    else if (IncludeFlag==10) {
      if ((c == ' ') &&
	  (Buff[BuffIdx-1] != ' ') && (Buff[BuffIdx-1] != '\n') &&
	  (BuffIdx > 0)) {
	Buff[BuffIdx++] = ' ';
      } else if ((c == '\n') &&
		 (Buff[BuffIdx-1] != '\n') &&
		 (BuffIdx > 0)) {
	Buff[BuffIdx++] = c;
      } else if ((c>31) && (c<127)) Buff[BuffIdx++]=c;
    }
  }
  Buff[BuffIdx - 1] = '\0';
  return Buff;
}

//////////////////////////////////////////////////////////////////////
//  StripAndTab - strips html tags off an html buffer and inserts
//     tabs where each tag goes, this aids in parsing later
//////////////////////////////////////////////////////////////////////
char * StripAndTab(const char * stringToStrip)
{
  char *Buff=(char *)malloc(strlen(stringToStrip));
  Buff[0]='\0';
  int buffLength = strlen(stringToStrip);
  int BuffIdx = 0;
  int u,c;
  int IncludeFlag = 5;
  bool tabFlag = FALSE;
  for ( u = 0; (u < buffLength);u++)
    {
      c = stringToStrip[u];
      if (c=='<') {
	tabFlag = FALSE;
	IncludeFlag=5;
      }
      else if (c=='>') {
	tabFlag = TRUE;
	IncludeFlag=10;
      }
      else if (IncludeFlag==10)
	{
	  if ( tabFlag ) {
            Buff[BuffIdx++] = '\t';
            tabFlag = FALSE;
         }
         if (((c>31) && (c<127)) || (c=='\n')) Buff[BuffIdx++]=c;
      }
   }
  Buff[BuffIdx - 1] = '\0';
  return Buff;
}                                                                                   
	    
void strip_commas( char *p )
{
   int i, j, len;

   len = strlen( p );
   for( i = 0, j = 0; i <= len; i++ )
       if( p[ i ] != ',' )
	   p[j++] = p[i];
   p[j] = '\0';
}
	    
//////////////////////////////////////////////////////////////////////////////////
// ProcessBidSubmission
//    takes the html response from submitting a bid and returns an appropriate
//    error code.
// return values:
//    1 - bid submission was a success - we are the high bidder
//    2 - Bid too low
//    3 - Quantity Problem
//    4 - Other problem (unknown, probably bad connection)
//   
//////////////////////////////////////////////////////////////////////////////////
int ProcessBidSubmission( char * Buff, char * lineBuff )
{
	istrstream streamBuff(Buff);
	bool foundKey = FALSE;
	bool foundEndOfForm = FALSE;
	bool foundAffirmation = FALSE;
	while ( streamBuff ) {
		streamBuff.getline(lineBuff, 1024, '\n');
		if ( stristr(lineBuff, "/FORM") != NULL ) {
			foundEndOfForm = TRUE;
			break;
		}
	}
	if ( !foundEndOfForm ) return ERROR;
	while ( streamBuff ) {
		streamBuff.getline(lineBuff, 1024, '\n');
		if ( strstr(lineBuff, "name=key") != NULL ) {
			foundKey = TRUE;
			if ( foundAffirmation ) return PBS_SUCCESS;
		}
		else if ( strstr(lineBuff, "Review bid for") != NULL
			  || strstr(lineBuff, "Review and confirm your bid:") != NULL )
		{
			foundAffirmation = TRUE;
			if (foundKey) return PBS_SUCCESS;
		}
		else if ( strstr(lineBuff, "Problem with bid amount") != NULL ) return PBS_BIDTOOLOW;
		else if ( strstr(lineBuff, "Problem with quantity") != NULL ) return PBS_BADQUANTITY;
		else if ( strstr(lineBuff, "Cannot proceed") != NULL ) return PBS_AUCTIONOVER;
	}
	return ERROR;
}
//////////////////////////////////////////////////////////////////////////////////
// ProcessBid()
//    takes the html response from making a bid
// return values:
//    1 - bid submission was a success - we are the high bidder
//    2 - bid successful, but we were outbid.
//    3 - bid under the current asking price
//    4 - problem with the bid quantity
//    5 - Auction is over.
//    6 - invalid username / password.
//    7 - unknown error
//   
//////////////////////////////////////////////////////////////////////////////////
int ProcessBid( char * Buff )
{
	char lineBuff[1024];
   istrstream streamBuff(Buff);
	bool foundEndOfForm = FALSE;
	while ( streamBuff ) {
		streamBuff.getline(lineBuff, 1024, '\n');
		if ( stristr(lineBuff, "/FORM") != NULL ) {
			foundEndOfForm = TRUE;
			break;
		}
	}
	if ( !foundEndOfForm ) {
	  printf("ProcessBid: Could not find end of form.\n");
	  return ERROR;
	}

	// This is way flawed.  There is no gaurantee of having each message
	// all on one line.
	/*
	while ( streamBuff ) {
		streamBuff.getline(lineBuff, 1024, '\n');

		if ( strstr(lineBuff, "are the current high bidder") != NULL )
		  return PB_HIGHBID;
		else if ( strstr(lineBuff, "outbid you") != NULL ) 
		  return PB_OUTBID;
		else if ( strstr(lineBuff, "You have been outbid") != NULL )
		  return PB_OUTBID;
		else if ( strstr(lineBuff, "Problem with bid amount") != NULL )
		  return PB_BIDTOOLOW;
		else if ( strstr(lineBuff, "Problem with quantity") != NULL )
		  return PB_BADQUANTITY;
		else if ( strstr(lineBuff, "Cannot proceed") != NULL )
		  return PB_AUCTIONOVER;
		else if ( strstr(lineBuff, "or password invalid") != NULL )
		  return PB_BADUSER;
	}
	*/

	if ( strstr(Buff, "are the current high bidder") != NULL )
	  return PB_HIGHBID;
	else if ( strstr(Buff, "outbid you") != NULL ) 
	  return PB_OUTBID;
	else if ( strstr(Buff, "You have been outbid") != NULL )
	  return PB_OUTBID;
	else if ( strstr(Buff, "Problem with bid amount") != NULL )
	  return PB_BIDTOOLOW;
	else if ( strstr(Buff, "Problem with quantity") != NULL )
	  return PB_BADQUANTITY;
	else if ( strstr(Buff, "Cannot proceed") != NULL )
	  return PB_AUCTIONOVER;
	else if ( strstr(Buff, "or password invalid") != NULL )
	  return PB_BADUSER;


	printf("Bid failed - unable to parse eBay's response. \n");
	printf("%s",Buff);
	return ERROR;
}
/*
//////////////////////////////////////////////////////////////////////////////////
// ParseEmailAddress
//    parses the email address out of the web paged passed in Html.
// return values:
//    1 - success
//    2 - authorization failure
//    2 - unknown error
//////////////////////////////////////////////////////////////////////////////////
int ParseEmailAddress(char * Buff, char * emailAddress)
{
   char tempBuff[1024];
   if ( strstr(Buff, "password invalid") != NULL ) return PEA_BADUSER;
   if ( strstr(Buff, "via e-mail:") == NULL ) return ERROR;
   // now the likely errors are weeded - so we can parse the 
   // email address out of Buff
   istrstream buffStream(Buff);
   while (buffStream)
   {
      buffStream.getline(tempBuff, 1024, '\n');
      if ( (strstr(tempBuff, "via e-mail:")) != NULL )
      {
         // parse the last little bit
         char * scratch;
         scratch = strtok(tempBuff,":");
         scratch = strtok(0,"\n");
         if ( scratch )
         {
            // now scrape off the leading space, there has to be a better way
            int limit = strlen(scratch);
            for ( int i =0; i < limit; i++ ) scratch[i] = scratch[i+1];
            strcpy(emailAddress, scratch);
         }
      }
   }
   return PEA_SUCCESS;
}

*/
///////////////////////////////////////////////////////////////////////////////
// ParseList
//   parses auction information from the listings or bidders list pages.
///////////////////////////////////////////////////////////////////////////////
int ParseList( auctioninfo ** TempList, char * Buff)
{
   char lineBuff[2048];
   int cnt;
   int cnt2;
   //   istrstream streamBuff(Buff);
   int listCount = 0;
   int nlpos;
   
   while( *Buff )
   {
     if(strchr(Buff,'\n')==NULL) nlpos=strlen(Buff)-1;
     else nlpos=(int)strchr(Buff,'\n')-(int)Buff;
     strncpy(lineBuff,Buff,nlpos);
     Buff+=nlpos+1;
     lineBuff[nlpos]='\0';
     if ( (lineBuff[0]=='\t') && (isdigit(lineBuff[1])) && (isdigit(lineBuff[2])) && 
	  (isdigit(lineBuff[3])) && (isdigit(lineBuff[4])) ) {
       cnt2 = 1;
       cnt=0;
       if ( strstr(lineBuff,"Hidden") == NULL )
         {
	   TempList[listCount]=new auctioninfo();
           char * scratch;
	   scratch = strtok(lineBuff, "\t");
	   if ( !scratch ) return listCount;
	   TempList[listCount]->ItemNumber=atoi(scratch);
	   
	   scratch = strtok(0,"\t");
	   if ( !scratch ) return listCount;
	   scratch = strtok(0,"\t");
	   if ( !scratch ) return listCount;
            strcpy(TempList[listCount]->Ends, scratch);

            scratch = strtok(0, "\t");
            if ( !scratch ) return listCount;
            TempList[listCount]->CurrentBid=atof(scratch);
	    //	    strip_commas( TempList[listCount]->CurrentBid );
	    
            scratch = strtok(0,"\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount]->Description, scratch);
	    
            scratch = strtok(0, "\t");
            if ( !scratch ) return listCount;
            strcpy(TempList[listCount]->HighBidder, scratch);


            listCount++;
            if (listCount >= (MAXAUCS - 1)) break;
         }
     }
   }
   return listCount;
}

// returns a 0-terminated array of ids of auctions in which we've participated
unsigned int * ParseList2(char * Buff)
{
  char lineBuff[2048];
  int nlpos;
  
  unsigned int ids[512], numids=0;
  unsigned int *retval;
  unsigned int i;
  
  
  while( *Buff )
    {
      // Get the next line
      if(strchr(Buff,'\n')==NULL) nlpos=strlen(Buff)-1;
      else nlpos=(int)strchr(Buff,'\n')-(int)Buff;
      strncpy(lineBuff,Buff,nlpos);
      Buff+=nlpos+1;
      lineBuff[nlpos]='\0';
      
      if ( (lineBuff[0]=='\t') && (isdigit(lineBuff[1])) && (isdigit(lineBuff[2])) && 
	   (isdigit(lineBuff[3])) && (isdigit(lineBuff[4])) ) {
	ids[numids++]=atoi(lineBuff+1);
	printf("Read id %d\n",ids[numids-1]);
      }
    }
  
  retval=(unsigned int *)malloc(sizeof(unsigned int)*(numids+1));
  for(i=0;i<numids;i++) retval[i]=ids[i];
  retval[i]=0;
  return retval;
}

//////////////////////////////////////////////////////////////////////////////
// CheckBadChars
// aids GetAucInfo in parsing html correctly
//////////////////////////////////////////////////////////////////////////////
void CheckBadChars(char *Str2Chk, char *GoodChars) {

  int idx,idx2;

  for (idx=0;Str2Chk[idx]!=0;++idx) {

    for (idx2=0;GoodChars[idx2]!=0;++idx2) {
      if (GoodChars[idx2]==Str2Chk[idx]) break; 
    }

    if (GoodChars[idx2]==0) { 
      Str2Chk[idx]=0; 
      return;
    }
  }
}
//////////////////////////////////////////////////////////////////////////////
// ReturnBidVarNum
//////////////////////////////////////////////////////////////////////////////
int ReturnBidVarNum(char *LineData, int auc_type)
{
  int idx=0;

  // Watch the change, it's strNcmp now.

  for(idx=0;SearchOps[auc_type][idx][0]!='*';++idx) 
    if (strncmp(LineData,SearchOps[auc_type][idx], strlen(SearchOps[auc_type][idx]))==0) {
      return (idx+1);
    }

  return 0;

}
//////////////////////////////////////////////////////////////////////////////
// ClearBidMakeUp:
// Clears the information in a BidMakeup struct
//////////////////////////////////////////////////////////////////////////////
void ClearBidMakeUp(struct auctioninfo * NewAuction) 
{
  NewAuction->ItemNumber=0;
  NewAuction->Description[0]=0;
  NewAuction->CurrentBid=0;
  NewAuction->FirstBid=0;
  NewAuction->Quantity=0;
  NewAuction->BidCount=0;
  NewAuction->TimeLeft[0]=0;
  NewAuction->Location[0]=0;
  NewAuction->Started[0]=0;
  NewAuction->Ends[0]=0;
  NewAuction->Seller[0]=0;
  NewAuction->HighBidder[0]=0;
  NewAuction->BidInc[0]=0;
  NewAuction->SellerRate[0] = 0;
  NewAuction->BidderRate[0] = 0;
  NewAuction->snipeAmount = 0;
  NewAuction->snipeQty = 0;
  NewAuction->myBidAmount=0;
  NewAuction->myBidQuantity=0;
  NewAuction->snipeKey[0] = 0;
  NewAuction->stat = '0';
  NewAuction->reserveMet = '0';
  NewAuction->isSnipe = FALSE;
  NewAuction->EndsValue = 0;
  if(NewAuction->infourl) delete NewAuction->infourl;
  if(NewAuction->infourl) delete NewAuction->bidurl;
  NewAuction->infourl=NULL;
  NewAuction->bidurl=NULL;

}
//////////////////////////////////////////////////////////////////////////////
// CalcEndsValue
//
// takes ebay format ending time (i.e. xx/xx/xx, xx:xx:xx) and converts
// it to time in seconds since a epoch (jan 1, 1970), this is for easy
// sorting later.
//
//////////////////////////////////////////////////////////////////////////////
long int CalcEndsValue( char * Ends )
{
   if ( strlen(Ends) > 12 )
   {
      //  convert Ends into in values of year, month,  day
      char tokEnds[50]; // the tokenized Ends, will be tossed.
      strncpy(tokEnds, Ends, 50);
      tokEnds[49] = '\0'; 
      char * scratch;
      int seconds,minutes,hours,month,day,year;
      scratch = strtok(tokEnds, "/");
      if ( scratch )  month = atoi(scratch);
      else return -1;
      scratch = strtok(0,"/");
      if ( scratch )  day = atoi(scratch);
      else return -1;
      scratch = strtok(0," ");
      if ( scratch )  year = atoi(scratch);
      else return -1;
      scratch = strtok(0,":");
      if ( scratch ) hours = atoi(scratch);
      else return -1;
      scratch = strtok(0,":");
      if ( scratch )  minutes = atoi(scratch);
      else return -1;
      scratch = strtok(0," \0");
      if ( scratch )  seconds = atoi(scratch);
      else return -1;
      if ( year < 25 ) year = year + 100;

      struct tm TheTime;
      TheTime.tm_sec = seconds;
      TheTime.tm_min = minutes;
      TheTime.tm_hour = hours;
      TheTime.tm_mday = day;
      TheTime.tm_mon = month - 1;
      TheTime.tm_year = year;
      TheTime.tm_yday = 0;
      TheTime.tm_wday = 0;
      TheTime.tm_isdst = -1;
         
      // now we have day,month,year,minutes,hours,seconds in integer format,
      // we just need to convert it to seconds since the epoch and we're in
      // business.
      long int the_time = mktime(&TheTime);
      return the_time;
   }
   else return -1; 
}
//////////////////////////////////////////////////////////////////////////////
//   CalcTimeLeft
//       calculates the time left in an auction in seconds.
//   stringTimeLeft
//       calculates the time left and return a string of that value
//////////////////////////////////////////////////////////////////////////////
long int CalcTimeLeft ( long int endTime, int ebayOffset )
{
   long int localTime = time(NULL);
   localTime += ebayOffset;
   return (endTime - localTime);
}
int stringTimeLeft ( long int endTime, int ebayOffset, char * chTimeLeft )
{
   long int localTime = time(NULL);
   localTime += ebayOffset;
   int intTimeLeft = (endTime - localTime);
   int err;
   if ( intTimeLeft >= 172800 ) {
      err = sprintf(chTimeLeft, "%d days, %d hrs", intTimeLeft / 86400,
                        (intTimeLeft % 86400) / 3600 );
		return intTimeLeft;
   }
   else if ( intTimeLeft >= 86400 ) {
      err = sprintf(chTimeLeft, "%d day, %d hrs", intTimeLeft / 86400,
                        (intTimeLeft % 86400) / 3600 );
		return intTimeLeft;
   }
   else if ( intTimeLeft >= 3600 ) {
      err = sprintf(chTimeLeft, "%d hrs, %d mins", intTimeLeft / 3600,
                        (intTimeLeft % 3600) / 60);
		return intTimeLeft;
   }
   else if ( intTimeLeft >= 60 ) {
      err = sprintf(chTimeLeft, "%d mins, %d secs", intTimeLeft / 60,
                        intTimeLeft % 60);
		return intTimeLeft;
   }
   else if ( intTimeLeft >= 0 ) {
      err = sprintf(chTimeLeft, "%d seconds", intTimeLeft );
      return intTimeLeft;
   }
   else {
      err = sprintf(chTimeLeft, "Auction Ended" );
      return intTimeLeft;
   }
}     
//////////////////////////////////////////////////////////////////////////////
//   translate_date
//       converts dates from "Jan-23-00 12:34:56" into "01/23/00, 12:34:56"
//////////////////////////////////////////////////////////////////////////////
void translate_date( char *date )
{
    int		i;
    char	*months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
			      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

    if( isdigit( *date ) )
	return;

    for( i = 0; i < 12; i++ )
    {
	if( strncasecmp( date, months[ i ], 3 ) == 0 )
	    break;
    }

    if( i >= 12 )
	return;

    if( date[ 3 ] != '-' || date[ 6 ] != '-' || date[ 9 ] != ' ' )
	return;

    sprintf( date, "%02d", i+1 );
    date[ 2 ] = '/';
    date[ 3 ] = date[ 4 ];
    date[ 4 ] = date[ 5 ];
    date[ 5 ] = '/';
    date[ 6 ] = date[ 7 ];
    date[ 7 ] = date[ 8 ];
    date[ 8 ] = ',';

}

// Parse out currency and price
void parseprice(char *buf,struct auctioninfo * auc, bool isfirst) {
  int len;
  if ((buf[0]=='$') || (buf[0]=='U' && buf[1]=='S')) strcpy(auc->currency,"$");
  else {
    strncpy(auc->currency,buf,6);
    auc->currency[6]='\0';
    len=strcspn(auc->currency," ");
    auc->currency[len]='\0';
  }
  len=strcspn(buf,"0123456789");
  buf+=len; // skip to price
  len=strspn(buf,"0123456789.,");
  buf[len]='\0';
  strip_commas(buf );
  
  if(isfirst) auc->FirstBid=atof(buf);
  else auc->CurrentBid=atof(buf);

  //printf("Price is '%s'\n",buf);
}

//////////////////////////////////////////////////////////////////////////////
//   GetAucInfo
//       loads up BidMakeup[] with auction info.
//////////////////////////////////////////////////////////////////////////////
bool auctioninfo::parseaucinfo(char *Buff)
{
   int idx=0;
   int cnt;
   int auc_type=TYPE_EBAY;
   char LineData[1025];
   char * scratch;
   char newName[76];

   // First Lets clear out the right bits of the structure
   //   memset( CurrentBid, 0, sizeof( CurrentBid ) );
   /* memset( FirstBid, 0, sizeof( FirstBid ) );
   memset( Quantity, 0, sizeof( Quantity ) );
   memset( BidCount, 0, sizeof( BidCount ) ); */
   memset( TimeLeft, 0, sizeof( TimeLeft ) );
   memset( Location, 0, sizeof( Location ) );
   memset( Started, 0, sizeof( Started ) );
   memset( Ends, 0, sizeof( Ends ) );
   memset( Seller, 0, sizeof( Seller ) );
   memset( HighBidder, 0, sizeof( HighBidder ) );
   reserveMet = 'x';
   
   CurrentBid = 0;  // Not resetting this breaks the logic of updating the bid
   BidCount = 0;    // This too.

   /*
    * Parse the description out of the buffer first. This is
    * most easily done at the buffer-level and not as we try
    * to read the buffer in a line-oriented manner. There is
    * probably a need to re-write this parser all together,
    * but that's not what I'm going to do right now.
    * Thanks to Bob Beaty!
    */
   scratch = strstr(Buff, ") -");
   if (scratch != NULL) {
     // move past the ") -"
     scratch += 3;
     // move past any whitespace
     while (isspace(*scratch)) scratch++;
     // copy over the description to a newline
     idx = 0;
     while (*scratch != '\n') {
       Description[idx++] = *scratch++;
     }
     // NULL terminate the description I just parsed off
     Description[idx] = '\0';
   } else {
     return FALSE;
   } 

   istrstream streamBuff(Buff);
   /* Skip everything before the start of auction data */
   
  while(strstr(LineData,"eBay")==NULL && streamBuff)
     streamBuff.getline(LineData, 1024, '\n');

  if ( strstr(LineData,"eBay item") == NULL )
    if (strstr(LineData,"eBay Motors item") == NULL)
      return FALSE;

  if (strstr(LineData,"eBay Motors item") != NULL) auc_type = TYPE_EBAYMOTORSCAR;

   while(strstr(LineData,") -")==NULL && streamBuff)
     streamBuff.getline(LineData, 1024, '\n');

   
   if (auc_type == TYPE_EBAYMOTORSCAR) {
     scratch = strstr(Buff, "First bid");
     if (scratch != NULL) {
       // This is totally crazy and will break as soon as they muck with
       // the html but it works for now to distinguish a Car auction from
       // a part or accessory.  Anyone want to offer a better suggestion?

       //Should move us ahead to a $.  This check is here to make sure
       //we aren't looking at something the seller said in his desc etc.
       scratch +=13;

       if (scratch[0] == '$') auc_type = TYPE_EBAYMOTORS;
     }
   }
   
   //printf("Auction Type=%i\n", auc_type);

   while (streamBuff) 
   {
      streamBuff.getline(LineData, 1024, '\n');
    
      // The Idea here is to get the info we can get, if for
      // some reason some of the data is missing or out of
      // order we can still salvage what we can..
    
      // This new case style I came up with is a bit redundant, but
      // it may allow for bidwatcher to grow into more types.  Like
      // Checking yahoo or something.  See also SearchOps. -kdwyer

      if (LineData[0]!=0) 
      {
	//if (ReturnBidVarNum(LineData, auc_type) != 0)
	//printf("DEBUG:%i\n", ReturnBidVarNum(LineData, auc_type));
	//printf("LD:%s:%i\n", LineData, strlen(LineData));
	switch (auc_type) {
	case TYPE_EBAY:
	  switch (ReturnBidVarNum(LineData, auc_type)) 
	    {
	    case 1: 
	    case 14: // "Lowest" -shows up on motorsnotcars
	    case 2: // current price
	      if (CurrentBid==0) {
		streamBuff.getline(LineData, 1024, '\n');
		parseprice(LineData,this,FALSE);

		streamBuff.getline(LineData, 1024, '\n');
		if (strstr(LineData,"reserve not") != NULL )
		  reserveMet = 'n';
		else if (strstr(LineData,"reserve met") != NULL )
		  reserveMet = 'y';
	     
		//		parseprice(LineData,this,FALSE);
		/*	     CheckBadChars(LineData,CheckPrice);
			     strncpy(CurrentBid,LineData,14); */
	      }
	      break;
	    case 3:  // first bid
	      if (FirstBid==0) {
		streamBuff.getline(LineData, 1024, '\n');
		parseprice(LineData,this,TRUE);
		/*	     CheckBadChars(LineData,CheckPrice);
			     strncpy(FirstBid,LineData,14); */
	      }
	      break;
	    case 4: // quantity
	      if (Quantity==0) {
		streamBuff.getline(LineData, 1024, '\n');
		CheckBadChars(LineData,"0123456789");
		//	     strncpy(Quantity,LineData,14);
		Quantity=atoi(LineData);
	      }
	      break;
	    case 5: // bid count
	      if (BidCount==0) {
		char bc[80];
		streamBuff.getline(LineData, 1024, '\n');
		for(cnt=0;(((LineData[cnt]!=' ') && 
			    (LineData[cnt]!=0)) && (cnt<14));++cnt)
		  bc[cnt]=LineData[cnt];
		bc[cnt]=0;
		BidCount=atoi(bc);
	      }
	      break;
	    case 6: // time left
	      if (TimeLeft[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(TimeLeft,LineData,50);
	      }
	      break;
	    case 7: // location
	      if (Location[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Location,LineData,100);
	      }
	      break;
	    case 8: // time started
	      if (Started[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Started,LineData,50);
		translate_date( Started );
	      }
	      break;
	    case 9: // time ending
	      if (Ends[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Ends,LineData,50);
		translate_date( Ends );
	      }
	      break;
	    case 10: // seller id
	      if (Seller[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Seller,LineData,75);
	      }
	      break;
	    case 11: // high bidder id
	      if (HighBidder[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(HighBidder,LineData,76);
		HighBidder[75] = '\0';
	      }
	      break;
	   
	      //case 12: idx=GetLineOfData(idx,1024,LineData,Buff);
	      //         CheckBadChars(LineData,CheckPrice);
	      //         strncpy(BidInc,LineData,14);
	      //         break;
	      
	    case 13: // dutch auction
	      streamBuff.getline(LineData, 1024, '\n');
	      if (strcmp(LineData, "Dutch Auction") == 0)
		strcpy(HighBidder, "Dutch Auction");
	      break;
	      
	    default:;
	    }
	  break;
	  /* End case 0 of auc_type */

	case TYPE_EBAYMOTORSCAR:
	  //printf("LD:%s:%i\n", LineData, strlen(LineData));
	  switch(ReturnBidVarNum(LineData,auc_type)) {
	  case 9:
	  case 1:  // Current Bid
	    if (CurrentBid==0) {
	      streamBuff.getline(LineData, 1024, '\n');
	      parseprice(LineData,this,FALSE);

	      streamBuff.getline(LineData, 1024, '\n');
	      if (strstr(LineData,"reserve not") != NULL)
		      reserveMet = 'n';
	      else if (strstr(LineData,"reserve met") != NULL)
		      reserveMet = 'y';
	      else if (strstr(LineData,"Time left") != NULL)
		      TimeLeft[0] = '0';
	    }
	    break;
	    
	  case 2: //Time left
	      if (TimeLeft[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(TimeLeft,LineData,50);
	      }
	    break;
	  case 3: //Seller
	      if (Seller[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Seller,LineData,75);
	      }
	    break;
	  case 4: //High bid
	      if (HighBidder[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(HighBidder,LineData,76);
		HighBidder[75] = '\0';
	      }
	    break;
	  case 5: //Number of bids
	      if (BidCount==0) {
		char bc[80];
		streamBuff.getline(LineData, 1024, '\n');
		for(cnt=0;(((LineData[cnt]!=' ') && 
			    (LineData[cnt]!=0)) && (cnt<14));++cnt)
		  bc[cnt]=LineData[cnt];
		bc[cnt]=0;
		BidCount=atoi(bc);
	      }
	      break;
	  case 6: //Location
	      if (Location[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Location,LineData,100);
	      }
	    break;
	  case 7: //Started
	      if (Started[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Started,LineData,50);
		translate_date( Started );
	      }
	    break;
	  case 8: //Ends
	      if (Ends[0]==0) {
		streamBuff.getline(LineData, 1024, '\n');
		strncpy(Ends,LineData,50);
		translate_date( Ends );
	      }
	    break;
	    
	  default:;
	    break;
	    /* End case 1 of auc_type */
	  }
	default:;
	  /* End of switch(auc_type) */
	}
      }
   }
   //  if ( CurrentBid == 0 ) strcpy(CurrentBid, "Error");

  // now calculate the ending time in seconds and save it
  
  
  // now We'll parse the High Bidder and Seller's ratings off their names
  // and assign them to .SellerRate and .BidderRate

   strcpy ( newName, Seller);

   scratch = strtok(newName, " ");

   if (!scratch) return FALSE;

   strcpy(Seller, scratch);
   scratch = strtok(0,")");

   if (scratch) {
      strcpy(SellerRate, scratch);
      strcat(SellerRate, ")");
   }

   if ((strcmp(HighBidder, "Dutch Auction") == 0) ||
       (strcmp(HighBidder, "see Dutch high bidders") == 0) ||
       (strcmp(HighBidder, "--") == 0) ) {
	   return TRUE;
   } else {
      strcpy(newName, HighBidder);
      scratch = strtok(newName, " ");

      if (!scratch) return FALSE;

      strcpy(HighBidder, scratch);
      scratch = strtok(0,")");
      if (scratch) {
          strcpy(BidderRate, scratch);
          strcat(BidderRate, ")");
      }
   }
   return TRUE;
} // end GetAucInfo()

void MakeFileName(char * prefix2, char * fileName, bool thisMonth)
{
  char * homedir=getenv("HOME");
  int month,year, err;
  time_t time_now = time(0);
  struct tm * timeStruct = localtime(&time_now);
  if ( thisMonth ) month = timeStruct->tm_mon + 1;
  else month = timeStruct->tm_mon;
  year = timeStruct->tm_year;
  if ( year > 99 ) year = year - 100;
  
  if ( year > 75 ) err = sprintf(fileName, "%s%s%d_%d.txt",
				 homedir, prefix2, month, year); 
  else err = sprintf(fileName, "%s%s%d_0%d.txt",homedir,prefix2, month, year); 
}
void MakeClockTime(int offSet, char * theTime)
{
   time_t intTime = time(NULL) + offSet;
   //struct tm tmTime = *localtime(&intTime);
   //int err = sprintf(theTime,"%d:%d:%d", tmTime.tm_hour, tmTime.tm_min, tmTime.tm_sec);
   //return;
   char * tempTime = ctime(&intTime);
   char * scratch = strtok(tempTime, " ");
   if ( !scratch ) return;
   scratch = strtok(0, " ");
   if ( !scratch ) return;
   scratch = strtok(0, " ");
   if ( !scratch ) return;
   scratch = strtok(0, " ");
   if ( !scratch ) return;
   strcpy(theTime, scratch);
   return;
}
//////////////////////////////////////////////////////////////////////////////
// calcTimeDiff
//  
//  buffer is an unfiltered webpage from ebay containing the 
//  current time (http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?TimeShow)
//
//  calcTimeDiff parses the time from the above mentioned page and 
//  calculates the difference between ebay time and local time, returning
//  the value in seconds.
//////////////////////////////////////////////////////////////////////////////
int calcTimeDiff ( char * buffer)
{  
   struct timeval time_tt;
   //time_t new_time_t = time(NULL);
   gettimeofday(&time_tt, 0);
   time_t new_time_t = time_tt.tv_sec;
   char chHr[9];
   char chMin[9];
   char chSec[9];
   char chYear[9];
   char chMonth[9];
   char chDay[9];
   char lineBuff[1024];
   char * scratch;
   istrstream streamBuff(buffer);
   struct tm ebay_time;
   ebay_time.tm_isdst = 0;
   while( streamBuff )
   {
      streamBuff.getline(lineBuff, 1024, '\n');
      if ( strstr(lineBuff, "Look here to see what time eBay thinks it is") != NULL )
      {
         scratch = strtok(lineBuff, ">");
         if (scratch) scratch = strtok(0, ">");
         if (scratch) scratch = strtok(0, ">");
         if (scratch) scratch = strtok(0, " ");
         if (scratch) scratch = strtok(0, " ");
         if (scratch) strcpy(chMonth, scratch);
         scratch = strtok(0, " ");
         if (scratch) strcpy(chDay, scratch);
         scratch = strtok(0, " ");
         if (scratch) strcpy(chYear, scratch);
	 scratch = strtok(0,": ");
	 if (scratch) strcpy(chHr, scratch);
	 scratch = strtok(0,":");
	 if (scratch) strcpy(chMin, scratch);
	 scratch = strtok(0, " ");
	 if (scratch) strcpy(chSec, scratch);
	 scratch = strtok(0, "<");
	 if (scratch && strcmp( scratch, "PDT" ) == 0 )
	     ebay_time.tm_isdst = 1;
         break;
      }
   }
   ebay_time.tm_hour = (((int(chHr[strlen(chHr) - 2])) - 48) * 10) +
                       ((int(chHr[strlen(chHr) - 1])) - 48);
   ebay_time.tm_min =  (((int(chMin[strlen(chMin) - 2])) - 48) * 10) +
                       ((int(chMin[strlen(chMin) - 1])) - 48);
   ebay_time.tm_sec =  (((int(chSec[strlen(chSec) - 2])) - 48) * 10) +
                       ((int(chSec[strlen(chSec) - 1])) - 48);
   ebay_time.tm_mday = atoi(chDay); 
   if ( strcmp("Jan", chMonth) == 0 ) ebay_time.tm_mon = 0 ;
   else if ( strcmp("Feb", chMonth) == 0 ) ebay_time.tm_mon = 1 ;
   else if ( strcmp("Mar", chMonth) == 0 ) ebay_time.tm_mon = 2 ;
   else if ( strcmp("Apr", chMonth) == 0 ) ebay_time.tm_mon = 3 ;
   else if ( strcmp("May", chMonth) == 0 ) ebay_time.tm_mon = 4 ;
   else if ( strcmp("Jun", chMonth) == 0 ) ebay_time.tm_mon = 5 ;
   else if ( strcmp("Jul", chMonth) == 0 ) ebay_time.tm_mon = 6 ;
   else if ( strcmp("Aug", chMonth) == 0 ) ebay_time.tm_mon = 7 ;
   else if ( strcmp("Sep", chMonth) == 0 ) ebay_time.tm_mon = 8 ;
   else if ( strcmp("Oct", chMonth) == 0 ) ebay_time.tm_mon = 9 ;
   else if ( strcmp("Nov", chMonth) == 0 ) ebay_time.tm_mon = 10 ;
   else ebay_time.tm_mon = 11;   
   ebay_time.tm_year = atoi(chYear) - 1900;  

   time_t theTime = mktime(&ebay_time) - new_time_t;

   return theTime;
}
//////////////////////////////////////////////////////////////////////////////
// parseRows
//   parses rows from raw html and puts them each on a line (if it
//   finds a newline character in a line it converts it to a space).
//////////////////////////////////////////////////////////////////////////////
char * parseRows(const char * fromBuff)
{
   // we have 2 states: (1) we are inside a row or (2) we are not.
   // to determine we are in a row we need to find "<td>", to determine
   // we are out we need to find </td>.
   bool inRow = FALSE;
   int bufflen = strlen(fromBuff);
   char * toBuffer=(char *)malloc(bufflen+1);
   int i,j;
   i = j = 0;

   bool inComment = false;
   for ( i=0; i<bufflen; i++) 
   {
        /*
         * Let's first delete all of the JavaScript code by removing anything
         * that is an html comment.
         */

        if ( inComment     == false &&
             fromBuff[i]   == '<' &&
             fromBuff[i+1] == '!' &&
             fromBuff[i+2] == '-' &&
             fromBuff[i+3] == '-' )
        {
            i+=3;
            inComment = true;
        }
        else if ( inComment     == true &&
                  fromBuff[i]   == '-' &&
                  fromBuff[i+1] == '-' &&
                  fromBuff[i+2] == '>' )
        {
            i+=2;
            inComment = false;
        }

        // If we're inside an HTML comment, throw chars out.
        if ( inComment == true )
        {
            continue;
        }

      if ( (inRow==FALSE) && 
           (fromBuff[i] == '<') && 
           (tolower(fromBuff[i + 1]) == 't') && 
           (tolower(fromBuff[i + 2]) == 'r') )
      {
         // we have found a cell and should start recording 
         inRow = TRUE;
         toBuffer[j++] = fromBuff[i];
      }
      else if ( (inRow==TRUE) && (fromBuff[i - 3] == '/') &&
		(tolower(fromBuff[i - 2]) == 't') && 
		(tolower(fromBuff[i - 1]) == 'r') &&
		(fromBuff[i] == '>'))
	{   
	  inRow = FALSE;
	  toBuffer[j++] = fromBuff[i];
	  toBuffer[j++] = '\n';
	}
      else if ( inRow==TRUE ) {
         // we will turn any newlines we find into spaces
         if ( fromBuff[i] == '\n' )
         {
             toBuffer[j++] = ' ';
         }
         else 
         {
             // not fond of carriage returns; throw them out.
             while ( fromBuff[i] == '\r' )
             {
                 i++;
             }
             toBuffer[j++] = fromBuff[i];
         }
      }
   }
   toBuffer[j] = '\0';
   return toBuffer;
}

   

 

//////////////////////////////////////////////////////////////////////////////
// encode_password
//   converts the password to use the %## sequence so that special characters
//   in the password don't cause problems with the URL.  (It also makes the
//   password a little more hidden on the disk, but that is barely worth 
//   mentioning)
//////////////////////////////////////////////////////////////////////////////
void encode_password( char *dest, const char *src )
{
    int		i, j;

    *dest = '\0';

    for( i = 0, j = 0; src[ i ] != '\0'; i++ )
	j += sprintf( &dest[ j ], "%%%2X", src[ i ] );
}

void decode_password( char *dest, const char *src )
{
    int		i, j;
    int		c, t;

    *dest = '\0';

    for( i = 0, j = 0; src[ i ] != '\0'; i++, j++ )
    {
	if( src[ i ] == '%' )
	{
	    c = 0;

	    t = src[ i+1 ];
	    if( t >= '0' && t <= '9' )
		c += t - '0';
	    else if( t >= 'a' && t <= 'f' )
		c += t - 'a' + 10;
	    else if( t >= 'A' && t <= 'F' )
		c += t - 'A' + 10;
	    else
	    {
		dest[ j ] = src[ i ];
		continue;
	    }

	    c *= 16;

	    t = src[ i+2 ];
	    if( t >= '0' && t <= '9' )
		c += t - '0';
	    else if( t >= 'a' && t <= 'f' )
		c += t - 'a' + 10;
	    else if( t >= 'A' && t <= 'F' )
		c += t - 'A' + 10;
	    else
	    {
		dest[ j ] = src[ i ];
		continue;
	    }

	    dest[ j ] = c;
	    i += 2;
	}
	else
	    dest[ j ] = src[ i ];
    }

    dest[ j ] = '\0';
}

// Lifted this one out of lazyness
// This is not particularly fast but it should do.
char *stristr(const char *haystack, const char *needle)
{
      char *pptr, *sptr, *start;
      uint  slen, plen;

      for (start = (char *)haystack,
           pptr  = (char *)needle,
           slen  = strlen(haystack),
           plen  = strlen(needle); slen >= plen; start++, slen--) {

            /* find start of pattern in string */
            while (toupper(*start) != toupper(*needle)) {
                  start++;
                  slen--;

                  /* if pattern longer than string */

                  if (slen < plen)
                        return(NULL);
            }

            sptr = start;
            pptr = (char *)needle;

            while (toupper(*sptr) == toupper(*pptr)) {
                  sptr++;
                  pptr++;

                  /* if end of pattern then pattern was found */

                  if ('\0' == *pptr)
                        return (start);
            }
      }
      return(NULL);
}
